"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
// import { join, dirname } from 'path';
const execa_1 = __importDefault(require("execa"));
const fs_1 = __importDefault(require("fs"));
const fs_extra_1 = require("fs-extra");
const util_1 = require("util");
const readFile = util_1.promisify(fs_1.default.readFile);
const writeFile = util_1.promisify(fs_1.default.writeFile);
const build_utils_1 = require("@now/build-utils");
const gem_install_1 = require("./gem-install");
async function gemInstall(gemPath, vendorDir, ...gems) {
    console.log(`running "gem install ${gems.join(' ')}"...`);
    try {
        await execa_1.default(gemPath, ['install', ...gems, '-​-no_document', '-​-install_dir', vendorDir], {
            stdio: 'inherit',
        });
    }
    catch (err) {
        console.log(`failed to run "gem install ${gems.join(' ')}"...`);
        throw err;
    }
}
// async function bundleInstall(bundlePath: string, workDir: string, ...args: string[]) {
//   console.log(`running "bundle ${args.join(' ')}"...`);
//   try {
//     await execa(bundlePath, args, {
//       cwd: workDir,
//       stdio: 'inherit',
//       env: {
//         'BUNDLE_SILENCE_ROOT_WARNING': '1',
//       }
//     });
//   } catch (err) {
//     console.log(`failed to run "bundle ${args.join(' ')}"...`);
//     throw err;
//   }
// }
exports.config = {
    maxLambdaSize: '5mb',
};
exports.build = async ({ workPath, files, entrypoint }) => {
    console.log('downloading files...');
    // eslint-disable-next-line no-param-reassign
    files = await build_utils_1.download(files, workPath);
    // this is where `ruby, gem and bundler` will be installed to
    // we need it to be under `/tmp`
    const gemHome = await build_utils_1.getWriteableDirectory();
    process.env.GEM_HOME = gemHome;
    const { gemPath } = await gem_install_1.downloadAndInstallBundler();
    // const { gemPath, bundlerPath } = await downloadAndInstallBundler();
    // reuse existing vendor directory if available
    const vendorFiles = await build_utils_1.glob('vendor/bundle/ruby/*', workPath);
    let vendorDir;
    if (Object.keys(vendorFiles).length) {
        vendorDir = vendorFiles[Object.keys(vendorFiles)[0]].fsPath;
    }
    else {
        vendorDir = path_1.join(workPath, 'vendor', 'bundle', 'ruby', '2.5.3');
    }
    await fs_extra_1.ensureDir(vendorDir);
    console.log('Using vendor dir', vendorDir);
    // install single requirement for http
    await gemInstall(gemPath, vendorDir, 'httparty');
    // const fsFiles = await glob('**', workPath);
    // const entryDirectory = dirname(entrypoint);
    // const gemFile = join(entryDirectory, 'Gemfile');
    // if (fsFiles[gemFile]) {
    //   console.log('found "Gemfile"');
    //   const gemFilePath = fsFiles[gemFile].fsPath;
    //   await bundleInstall(bundlerPath, workPath, 'package', '--no-install', '--gemfile', gemFilePath);
    // }
    const originalRbPath = path_1.join(__dirname, 'now_init.rb');
    const originalNowHandlerRbContents = await readFile(originalRbPath, 'utf8');
    // will be used on `from $here import handler`
    // for example, `from api.users import handler`
    console.log('entrypoint is', entrypoint);
    const userHandlerFilePath = entrypoint.replace(/\.rb$/, '');
    const nowHandlerRbContents = originalNowHandlerRbContents.replace(/__NOW_HANDLER_FILENAME/g, userHandlerFilePath);
    // in order to allow the user to have `server.rb`, we need our `server.rb` to be called
    // somethig else
    const nowHandlerRbFilename = 'now__handler__ruby';
    await writeFile(path_1.join(workPath, `${nowHandlerRbFilename}.rb`), nowHandlerRbContents);
    const lambda = await build_utils_1.createLambda({
        files: await build_utils_1.glob('**', workPath),
        handler: `${nowHandlerRbFilename}.handler`,
        runtime: 'ruby2.5',
        environment: {},
    });
    return {
        [entrypoint]: lambda,
    };
};
